/*

    svstat - Read SuperVozelj node telemetric statistics, and write a PNG
    image within a graph of the read values.
    Copyright (C) 2001  Paolo Subiaco

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
    -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

    svstat v0.3 - 01/04/2001 - (c) Paolo Subiaco
    
    Authors:
  	Paolo Subiaco iw3grx@ir3ip.ampr.org  http://www.ir3ip.ampr.org/iw3grx
  	
    Pourpose:	
	This software is for amateur radio only people.
	Read supervozelj node statistics like supply voltage, internal and 
	external temperatures, lux probes, etc; 
	archive those values into an archive, and write a PNG image withing 
	a graph.
	
    Compile:
    	make all
    	
    Require:
    	libgd	(installed in much of the newer linux distributions: you can
    		find it at http://www.boutell.com/gd/ URL)
    	
    	libax25 and ax25-tools
	
    Execute:
    	from the shell prompt, type
    	
    	$ svstat AX25_PORTNAME FIRSTNODE UTC_DIFF_SECONDS NODE1 [NODE2 [NODE3 ...]]
	
	where PORTNAME is the name of ax.25 port (defined in /etc/axports),
	FIRSTNODE is the first connectable node with SSID (like IR3TVE-11),
	UTC_DIFF_SECONDS is the difference in seconds between GMT and LOCAL time,
	and the next args are the node callsigns from which i would read 
	the telemetric values...
	For example:
	
	$ svstat kiss ir3tve-11 7200 ir3cv ir3mco ir6pst
*/

#include <sys/types.h>
#include <utime.h>
#include <stdio.h>
#include <ctype.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <pwd.h>
#include <signal.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <termios.h>
#include <unistd.h>
#include <curses.h>

#ifdef HAVE_NETAX25_AX25_H
#include <netax25/ax25.h>
#else
#include <netax25/kernel_ax25.h>
#endif
#ifdef HAVE_NETROM_NETROM_H
#include <netrom/netrom.h>
#else
#include <netax25/kernel_netrom.h>
#endif
#ifdef HAVE_NETROSE_ROSE_H
#include <netrose/rose.h>
#else
#include <netax25/kernel_rose.h>
#endif

#include <netax25/axlib.h>
#include <netax25/axconfig.h>
#include <netax25/nrconfig.h>
#include <netax25/rsconfig.h>




#define	CTRL_C		0x03

#define	MAX_PACKETLEN	512
#define	MAX_BUFLEN	2*MAX_PACKETLEN
#define	MAX_CMPSTRLEN	MAX_PACKETLEN

#define	STD_DWN_DIR	"/var/spool/ax25/"

#define	FLAG_RECONNECT	0x01

#define	STATW_BITS	12
#define	STATW_STAT	20


#include <gd.h>
#include <gdfontg.h>
#include <gdfonts.h>
#include <gdfontmb.h>
#include <math.h>

#define LOG(format, arg) sprintf (buffer, format, arg); write( fdlog, buffer, strlen (buffer))

#define FILE_LOG "./svstat.log"	/* Log file */		
#define MAXREAD	2048     				/* Max RX buffer */
#define CONN_TIMEOUT 60  				/* Max 60 seconds to connect a node */
#define CONN_WAIT 1							/* Seconds between connections */

static int backoff = -1;
static int ax25mode = -1;

static int window = 0;
static char *port = NULL;

int interrupted = FALSE;
int paclen = 0;
int fd,fdlog,fddat;
int nread,i,j,k,l,m;
char buffer[MAXREAD];
int buflen,bufcnt;	// used by waitstring function
char line[MAXREAD];
char filename[32];
float value[11];		// telemetric values
char exist[11]; 		// mark the existing of each sensor (don't produce graph of unused A/D channel)
time_t start_time;  // local time in seconds
struct tm *lt;
int diff_time;	    // difference between local and gmt time
char nodeprompt[12];
int ssid=13;

static int connect_to(char *address)
{
  int fd = 0;
  int addrlen = 0;
  union {
    struct full_sockaddr_ax25 ax25;
    struct sockaddr_rose rose;
  } sockaddr;

  if (address == NULL) {
    fprintf(stderr, "call: too few arguments for AX.25\n");
    return (-1);
  }
  if ((fd = socket(AF_AX25, SOCK_SEQPACKET, 0)) < 0) {
    perror("socket");
    return (-1);
  }
  ax25_aton(ax25_config_get_addr(port), &sockaddr.ax25);
  sockaddr.ax25.fsa_ax25.sax25_family = AF_AX25;
  addrlen = sizeof(struct full_sockaddr_ax25);
  
  if (setsockopt(fd, SOL_AX25, AX25_WINDOW, &window, sizeof(window)) == -1) {
    perror("AX25_WINDOW");
    close(fd);
    return (-1);
  }
  if (setsockopt(fd, SOL_AX25, AX25_PACLEN, &paclen, sizeof(paclen)) == -1) {
    perror("AX25_PACLEN");
    close(fd);
    return (-1);
  }
  if (backoff != -1) {
    if (setsockopt(fd, SOL_AX25, AX25_BACKOFF, &backoff, sizeof(backoff)) == -1) {
      perror("AX25_BACKOFF");
      close(fd);
      return (-1);
    }
  }
  if (ax25mode != -1) {
    if (setsockopt(fd, SOL_AX25, AX25_EXTSEQ, &ax25mode, sizeof(ax25mode)) == -1) {
      perror("AX25_EXTSEQ");
      close(fd);
      return (-1);
    }
  }

  if (bind(fd, (struct sockaddr *) &sockaddr, addrlen) == -1) {
    perror("bind");
    close(fd);
    return (-1);
  }
  if (ax25_aton((const char*)address, &sockaddr.ax25) == -1) {
    close(fd);
    puts("Errore funzione ax25_aton_arglist");
    return (-1);
  }
  addrlen = sizeof(struct full_sockaddr_ax25);



  if (connect(fd, (struct sockaddr *) &sockaddr, addrlen)) {
    printf("\n");
    perror("connect");
    close(fd);
    return (-1);
  }
  return (fd);
}

int waitstring(char *string) {
  time_t watchdog;
  int i=0;

	
  line[0]=0;
  watchdog=time(NULL);

  while (1) {
    usleep(500000);
    while ((buflen=read(fd,buffer,MAXREAD-1))>0)
      {
	// parse the chars arrived
	for (bufcnt=0;bufcnt<buflen;i++,bufcnt++)
	  { line[i]=buffer[bufcnt];
	  if (buffer[bufcnt]=='\r' || buffer[bufcnt]=='\n' || i==MAXREAD-1) {
	    // new line!! parse this line.
	    line[i]=0;
	    fflush(stdout);
	    if (strstr(line,string)) {
	      bufcnt++;
	      return 1; // line found: return true!
	    }
	    i=-1;
	  }
	  }
      }
    if ((time(NULL)-watchdog)>CONN_TIMEOUT)
      {
	return 0;
      }
  }	
}			

void updatedat(char *callsign)
{	//write telemetry to the dat file, and update the graphs within statistcs
  FILE *stim;
  gdImagePtr im[11];
  int white,lightgrey,black,blue,red,magenta,marker;
  int i,j,k,xold,xnew,bfn; //bfn=index where basefilename finish.
  int style[32];
  time_t time,timeold;
  struct telem
  { int timepx;  		// time in pixel (seconds*16/3600=seconds/225)
    float val[11];  // power supply voltage,internal temperature,external temperature	
  } misure[300];
	
  struct
  {	int maxt;			// time where there was a max
    int mint;
    float maxv;   // value of max
    float minv;	
  } bound[11];
  gdPoint poly[11][304];
	
  float maxv[11],minv[11];	
  int nchannel=0;   // number of channels (defined into dat file)
  int record_len=0; // lenght of each record of dat file	
	
	
  strcpy(filename,"dat/");
  strcat(filename,callsign);
	
  for (bfn=0;filename[bfn];bfn++)
    filename[bfn]=tolower(filename[bfn]);
	
  strcpy(filename+bfn,".dat");
  if ((fddat=open(filename,O_RDWR|O_CREAT, 0644))>0)
    {	// write the dat/callsign.dat file
      read(fddat,buffer,32);
      if (strncmp(buffer,"svstat v",8))
	{ close (fddat);
	unlink(filename);
	if ((fddat=open(filename,O_RDWR|O_CREAT|O_TRUNC, 0644))<0)
	  { LOG("Error re-creating archive file %s\n",filename);
	  return;
	  }
	write (fddat,"svstat v0.3 iw3grx  nchannel=11",32);
	}
      lseek(fddat,0,SEEK_SET);
      read(fddat,buffer,32);
      sscanf(strstr(buffer,"nchannel=")+9,"%u",&nchannel);
      if (nchannel<=0 || nchannel>11)
	{	LOG("Archive file %s corrupted (incongruent number of channels)\n",filename);
	close(fddat);
	return;		
	}
      record_len=sizeof(time)+nchannel*sizeof(float);
		
      lseek(fddat,0,SEEK_END);
      write(fddat,&start_time,sizeof(start_time));
      write(fddat,value,nchannel*sizeof(float));

      // and now rebuild the daily graph image
      im[0]=gdImageCreate(540,150);
      white=gdImageColorAllocate(im[0],255,255,255);
      lightgrey=gdImageColorAllocate(im[0],232,232,232);
      black=gdImageColorAllocate(im[0],16,16,16);
      blue=gdImageColorAllocate(im[0],0,0,240);
      red=gdImageColorAllocate(im[0],255,0,0);
      magenta=gdImageColorAllocate(im[0],255,0,255);
      marker=gdImageColorAllocate(im[0],0,128,80);
      gdImageFilledRectangle(im[0],0,0,539,149,lightgrey);
      gdImageLine(im[0],0,149,539,149,black);
      gdImageLine(im[0],539,149,539,0,black);
      gdImageFilledRectangle(im[0],96,30,480,130,white);
      gdImageRectangle(im[0],96,30,480,130,black);
      sprintf(buffer,"SuperVozelj telemetry: %s",callsign);
      gdImageString(im[0],gdFontGiant,158,1,buffer,blue);
      gdImageStringUp(im[0],gdFontGiant,2,102,"Daily",blue);
      gdImageStringUp(im[0],gdFontGiant,16,125,"statistics",blue);

      for (j=0;j<25;j++)
    	style[j]=gdTransparent;
      style[0]=black;
      style[1]=black;
      style[2]=black;
      style[7]=black;
      style[8]=black;
      style[9]=black;
      style[10]=black;
      style[15]=black;
      style[16]=black;
      style[17]=black;
      style[18]=black;
      style[23]=black;
      style[24]=black;
      gdImageSetStyle(im[0],style,25);
      for (j=1;j<12;j++)
	{	gdImageLine(im[0],96+32*j,30,96+32*j,129,gdStyled);
	gdImageLine(im[0],96+32*j,130,96+32*j,134,black);
	sprintf(buffer,"%d",j*2);
	gdImageString(im[0],gdFontSmall,92+32*j,135,buffer,black);
	}
      style[14]=black;
      gdImageSetStyle(im[0],style,16);
      for (j=1;j<4;j++)
	{	gdImageLine(im[0],96,30+j*25,479,30+j*25,gdStyled);
	gdImageLine(im[0],92,30+j*25,96,30+j*25,black);
	}
      gdImageLine(im[0],2,6,30,6,magenta);
      gdImageString(im[0],gdFontSmall,32,0,"Yeasterday",magenta);
      gdImageLine(im[0],2,16,30,16,red);
      gdImageString(im[0],gdFontSmall,32,10,"Today",red);
      gdImageLine(im[0],2,16,30,16,red);
      gdImageString(im[0],gdFontSmall,90,135,"Time:",black);
      gdImageLine(im[0],2,16,30,16,red);
      gdImageString(im[0],gdFontSmall,457,135,"(local)",black);
      sprintf(buffer,"%02d/%02d/%02d",lt->tm_mday,lt->tm_mon+1,lt->tm_year%100);
      gdImageString(im[0],gdFontMediumBold,5,136,buffer,black);
		
      for (i=1;i<nchannel;i++)
	{ if (!exist[i])
	  continue;
	im[i]=gdImageCreate(540,150);
	gdImageColorAllocate(im[i],255,255,255);
	lightgrey=gdImageColorAllocate(im[i],232,232,232);
	gdImageColorAllocate(im[i],16,16,16);
	gdImageColorAllocate(im[i],0,0,240);
	gdImageColorAllocate(im[i],255,0,0);
	gdImageColorAllocate(im[i],255,0,255);
	gdImageColorAllocate(im[i],0,128,80);
	gdImageFilledRectangle(im[i],0,0,539,149,lightgrey);
	gdImageCopy(im[i],im[0],0,0,0,0,540,150);
	}
		
      // scan the archive adding each telemetry value on misure[] struct, ordered by time
      timeold=start_time+1;
      for (j=0,k=0-record_len;lseek(fddat,(off_t)k,SEEK_END)>=32 && j<300;j++,k-=record_len) { 
	read(fddat,&time,sizeof(time));
	read(fddat,&(misure[j].val),nchannel*sizeof(float));
	if (start_time-time>86400)
	  break;
	else if (time>=timeold)
	  { j--;
	  continue;
	  }
	else
	  timeold=time;
	misure[j].timepx=(time%86400)/225;
	if (j==0) { 
	  for (i=0;i<nchannel;i++) { 
	    if (!exist[i]) continue;
	    bound[i].minv=misure[j].val[i];
	    bound[i].maxv=misure[j].val[i];
	    bound[i].mint=misure[j].timepx;
	    bound[i].maxt=misure[j].timepx;
	  }
	}
	else { 
	  for (i=0;i<nchannel;i++) { 
	    if (!exist[i]) continue;
	    if (misure[j].val[i]<bound[i].minv) {
	      bound[i].minv=misure[j].val[i];
	      bound[i].mint=misure[j].timepx;
	    }
	    else if (misure[j].val[i]>bound[i].maxv) {
	      bound[i].maxv=misure[j].val[i];
	      bound[i].maxt=misure[j].timepx;
	    }
	  }
	}
	//	printf("value[0][%d]=%+.2f\n",j,misure[j].val[0]); 
      }
      close(fddat);
      // mark the max and min bound time
      for (i=0;i<nchannel;i++)
	{ // calculate x position of time labels
	  if (!exist[i]) continue;
	  xold=bound[i].mint;  // min time
	  xnew=bound[i].maxt;  // max time
	  if (abs(xnew-xold)<35)	
	    { k=(35-abs(xnew-xold))/2;	
	    if (xnew<xold)
	      k=-1*k;
	    xnew+=k;
	    xold-=k;
	    }
	  gdImageLine(im[i],bound[i].mint+96,27,bound[i].mint+96,129,marker);
	  sprintf(buffer,"%02d:%02d",bound[i].mint/16,(bound[i].mint%16)*60/16);
	  gdImageString(im[i],gdFontSmall,xold+81,15,buffer,marker);
	  gdImageLine(im[i],bound[i].maxt+96,27,bound[i].maxt+96,129,marker);
	  sprintf(buffer,"%02d:%02d",bound[i].maxt/16,(bound[i].maxt%16)*60/16);
	  gdImageString(im[i],gdFontSmall,xnew+81,15,buffer,marker);
	}		
      xold=misure[--j].timepx;
      for (i=0;i<nchannel;i++)
	{ if (!exist[i]) continue;
	minv[i]=bound[i].minv;
	maxv[i]=bound[i].maxv;
	if ((maxv[i]-minv[i])<1.0)
	  {	maxv[i]+=0.5;
	  minv[i]-=0.5;
	  }
	minv[i]=floorf(minv[i]-0.2*(maxv[i]-minv[i]));
	maxv[i]=floorf(maxv[i]+1+0.2*(maxv[i]-minv[i]))-minv[i];
	}
      for (i=0;i<nchannel;i++)	
	{ if (!exist[i]) continue;
	for (xnew=0;xnew<=4;xnew++)
	  {	
	    sprintf(buffer,"%+7.2f",minv[i]+maxv[i]/4*xnew);
	    gdImageString(im[i],gdFontSmall,50,123-xnew*25,buffer,black);
	  }
	maxv[i]=100/maxv[i];
	}
      for (k=0;j>=0;j--,k++)
	{	xnew=misure[j].timepx;
	if (xnew<xold)	// today, not yeasterday: draw yeasterday polygon
	  { if (xnew<950) // near the midnight?
	    { for (i=0;i<nchannel;i++)
	      {	if (!exist[i]) continue;
	      poly[i][k].x=480;
	      poly[i][k].y=130-(int)((misure[j].val[i]-minv[i])*maxv[i]);
	      }
	    k++;					
	    }
	  for (i=0;i<nchannel;i++)
	    { if (!exist[i]) continue;
	    poly[i][k].x=poly[i][k-1].x;
	    poly[i][k].y=129;
	    poly[i][k+1].x=poly[i][0].x;
	    poly[i][k+1].y=129;
	    gdImageFilledPolygon(im[i],poly[i],k+2,magenta);
	    }
	  k=0;
	  for (i=0;i<nchannel;i++)
	    { if (!exist[i]) continue;
	    poly[i][k].x=96;
	    poly[i][k].y=130-(int)((misure[j].val[i]-minv[i])*maxv[i]);
	    }
	  }
	else
	  {	for (i=0;i<nchannel;i++)
	    { if (!exist[i]) continue;
	    poly[i][k].x=xnew+96;
	    poly[i][k].y=130-(int)((misure[j].val[i]-minv[i])*maxv[i]);
	    }
	  }
	xold=xnew;
	}					
      for (i=0;i<nchannel;i++)
	{ if (!exist[i]) continue;
	poly[i][k].x=poly[i][k-1].x;
	poly[i][k].y=129;
	poly[i][k+1].x=poly[i][0].x;
	poly[i][k+1].y=129;
	gdImageFilledPolygon(im[i],poly[i],k+2,red);
	}		
		
      // mark the max and min bound values
      for (i=0;i<nchannel;i++)
	{ if (!exist[i]) continue;
	j=130-(int)((bound[i].minv-minv[i])*maxv[i]);
	gdImageLine(im[i],bound[i].mint+93,j,485,j,marker);
	sprintf(buffer,"%+.1f",bound[i].minv);
	gdImageString(im[i],gdFontSmall,484,j-2,buffer,marker);
	j=130-(int)((bound[i].maxv-minv[i])*maxv[i]);
	gdImageLine(im[i],bound[i].maxt+93,j,485,j,marker);
	sprintf(buffer,"%+.1f",bound[i].maxv);
	gdImageString(im[i],gdFontSmall,484,j-11,buffer,marker);
	}		
		
      for (i=0;i<nchannel;i++)
	{
	  if (!exist[i]) continue;
	  gdImageInterlace(im[i],0);
	  // Now save the graph	
	  sprintf(filename+bfn,"-%ddaily.png",i+1);
	  stim=fopen(filename,"w+");
	  gdImagePng(im[i],stim);
	  fclose(stim);
	  gdImageDestroy(im[i]);
	}
      buffer[0]=0; // silly solution to avoid a bug writing log file
    }
  else
    LOG("Error opening archive file %s\n",filename);
}

int main(int argc, char **argv) {
	
  if (argc<5)
    { printf("Usage: %s PORTNAME CALLSIGN diffTIME NODE1 [NODE2 [NODE3 ...]]\nFor example: %s 70cm ir3csm 7200 ir3cv ir3mco\n",argv[0],argv[0]);
    return -1;
    }
	
  diff_time=0;
  diff_time=atoi(argv[3]);


  start_time=time(NULL);
  lt=localtime(&start_time);
  // open log file
  if (!(fdlog=open(FILE_LOG,O_WRONLY|O_CREAT|O_APPEND,0640)))
    { strcpy(buffer,"Error opening output file ");
    strcat(buffer,FILE_LOG);
    perror(buffer);
    return -1;
    }
  sprintf(buffer,"\n%s (difftime=%d): svstat program is starting....\n",ctime(&start_time),diff_time);
  start_time-=diff_time;	//now start_time will be the local time.
  for(i=1;buffer[i]!='\n' && i<30;i++);
  if (buffer[i]=='\n')
    buffer[i]=' ';
  LOG(buffer,0);

  port = argv[1];
  if (ax25_config_load_ports() == 0) {
    fprintf(stderr, "svstat: no AX.25 port data configured\n");
    return 1;
  }
  if (window == 0)
    window = ax25_config_get_window(port);
  if (paclen == 0)
    paclen = ax25_config_get_paclen(port);

  for (i=0;i<strlen(argv[2]);i++)
    argv[2][i]=toupper(argv[2][i]);

  if ((fd=connect_to(argv[2]))==-1) {
    printf("Error connecting %s\n",argv[2]);
    return -1;
  }
  fcntl(fd,F_SETFL,O_NONBLOCK); //set the socket in non-blocking mode
  /* connect_to() return fd=socket descriptor */
	
  for (j=2;j<argc;j++)
    { if (j==3) j++; //argv[3] is not a callsign (diff_time)
      for (i=0;i<6 && argv[j][i]>'0' && argv[j][i]<'z';i++)	// get the callsign in uppercase without SSID
	argv[j][i]=toupper(argv[j][i]);
      argv[j][i]=0;
    }
	
  if (waitstring(argv[2]))
    { // Ok, i got the connection to the first node.
      for (i=4;i<argc;i++)	// for every node specified as parameter....
	{ //read the telemetric statistics
	  if (++ssid>15) // use ssid between 13 and 15 to disable reconnect feature.
	    ssid=13; 			//ssid 12 reserved for GLEJ linuxnode command
	  usleep(CONN_WAIT*1000000); // wait some seconds between connections
	  sprintf(buffer,"C %s-%d\r",argv[i],ssid);
	  write(fd,buffer,strlen(buffer));
	  sprintf(nodeprompt,"%s>",argv[i]);
	  if (!waitstring(nodeprompt)) // connection to the next node impossible
	    {	// now i must ensure that i'm connected to the first node....
	      LOG("Connection to %s impossible! Try to return to the first node.\n",argv[i]);
	      write(fd," \r",1);
	      usleep(1000000);
	      write(fd," \r",1);
	      usleep(1000000);
	      read(fd,buffer,MAXREAD); // destroy rx buffer
	      write(fd," \r",1);
	      sprintf(nodeprompt,"%s>",argv[2]);
	      if (!waitstring(nodeprompt))
		{
		  write(fd,"Q\r",2);
		  sprintf(nodeprompt,"%s>",argv[2]);
		  if (!waitstring(nodeprompt))
		    { // Error: i don't know how to return to the argv[2] node.
		      LOG("Sorry, I don't know how to return to the first %s node: EXIT!\n",argv[2]);
		      break;
		    }
		}
				// .... and then start new connection to the next node
	      continue;
	    }		  	
	  else // ok, i'm connected to the argv[i] node
	    { // read the telemetric statistics
	      LOG("%6s: ",argv[i]);
	      write(fd,"E\r",2);
	      if (waitstring("1:"))
		{ // which telemetric card is installed?
		  if (!strncmp(line,"#0 1:",5)) // original telemetric board?
		    {
		      if (!strstr(line,argv[i])) // verify callsign after telemetric values....
			{ // these telemetric values does NOT come from argv[i] node!
			  LOG("Read telemetric values from the non-selected node: %s\n",line);
			}
		      else
			{	LOG("%s\n",line);
			for (m=0;m<11;m++)
			  { exist[m]=0;
			  value[m]=0;
			  }
			// now store the 3 values in the archive file
			for(m=0,k=0,l=strlen(line);m<3;m++)
			  {		  		
			    for (j=k;j<l && !strncmp(line+j,"1:",2);j++);
			    for (;line[j]!='/' && j<l;j++);
			    for (j++,k=j+1;k<l && (line[k]=='.' || isdigit(line[k]));k++);
			    line[k++]=0;
			    sscanf(line+j,"%f",&value[m]);
			  }
			exist[0]=1; exist[1]=1; exist[2]=1;
			// telemetric values stored in value[0..2]
			// Store telemetric values into "argv[i].dat" file and update telemetry graphs
			updatedat(argv[i]);
			}
		    }
		  else if (!strncmp(line,"#1: (",5)) // iw3grx telemetric board?
		    { for (m=0;m<11;m++)	// initialize the 11 values
		      { value[m]=0;
		      exist[m]=0;			  			
		      }
		    for (m=0;m<11;m++) // read 11 values (in 11 lines)
		      { if (line[0]=='#' && (!strncmp(line+2,": (",3) || !strncmp(line+3,": (",3)))
			{ // ok, line correct
			  if (atoi(line+1)!=m+1)
			    m=atoi(line+1)-1;
			  l=strlen(line);
			  for (j=8;line[j]!=')' && j<l;j++);
			  j+=2;
			  for (k=j+1;line[k]!=' ' && k<l;k++);
			  if (k<l)
			    line[k]=0;
			  else
			    line[l-1]=0;
			  sscanf(line+j,"%f",&value[m]);
			  if (!strstr(line+k+1,"None"))
			    exist[m]=1;
			}
		      else
			break;
		      if (bufcnt<buflen) // read the next line from buffer
			{ for (;(buffer[bufcnt]=='\r' || buffer[bufcnt]=='\n') && bufcnt<buflen;bufcnt++);
			for (j=0;(buffer[bufcnt]!='\r' && buffer[bufcnt]!='\n') && bufcnt<buflen && j<sizeof(line)-1;bufcnt++,j++)
			  line[j]=buffer[bufcnt];
			line[j]=0;	
			bufcnt++;
			}
		      else
			if (!waitstring("#"))
			  break;		  						
		      }
		    
		    // control that,in the next line,the prompt callsign==argv[i], and then
		    // write values to the argv[i].dat file.
		    if (strstr(line,argv[i]) || 1) //WARNING: SuperVozelj software must be modified to show only the first 11 channels, no more! Then, i must leave "|| 1" from this instruction 
		      {
			line[0]=0;
			for (m=0;m<11;m++)
			  if (exist[m])
			    sprintf(line+strlen(line),"%d:%.2f ",m+1,value[m]);
			LOG("%s\n",line);
			updatedat(argv[i]);		  		
		      }
		    }
		  else // error getting telemetry
		    {
		      LOG("Impossible to read telemetric values!%s\n","");
		    }
		  for (k=0,j=0;j<10;j++) // where am i connected?  (k=1 if i sent Q command) { 
		    usleep(1000000);
		    read(fd,buffer,MAXREAD);	// emty buffer!
		    write(fd,"\r",1);	
		    waitstring(">");	// read node prompt
		    if (!k && strstr(line,nodeprompt)) { 
		      write(fd,"Q\r",2);
		      usleep(2000000);
		      j=0;
		      k=1; // command Q sent: disconnecting....
		    }
		    else if (strstr(line,argv[2]))
		      break;	
		    }	
	      // verify that i'm connected with the first node...
	      usleep(1000000);
	      read(fd,buffer,MAXREAD);
	      write(fd,"\r",1);
	      sprintf(nodeprompt,"%s>",argv[2]);
	      if (!waitstring(nodeprompt))
		{ //error disconnecting node argv[i]
		  LOG("Error disconnecting node %s: exit\n",argv[i]);
		  break;		
		}
	      
	    }
	}
    }
  else
    {	// Error connecting the first node! exit
      LOG("Error connecting the fist node %s\n",argv[2]);
    }
  // now disconnect to the first node!
  write(fd,"Q\r",2);
  usleep(1000000);
  close(fd);
  close(fdlog);
  return 0;
}
